@@ -2,12 +2,19 @@
 
 namespace Icinga\Module\Director\Controllers;
 
-use dipl\Html\Html;
-use dipl\Html\Link;
-use dipl\Web\Url;
+use gipfl\Web\Widget\Hint;
+use Icinga\Module\Director\Auth\Permission;
+use Icinga\Module\Director\Integration\Icingadb\IcingadbBackend;
+use Icinga\Module\Director\Integration\MonitoringModule\Monitoring;
+use Icinga\Module\Director\Web\Table\ObjectsTableService;
+use ipl\Html\Html;
+use gipfl\IcingaWeb2\Link;
+use gipfl\IcingaWeb2\Url;
+use gipfl\IcingaWeb2\Widget\Tabs;
 use Exception;
 use Icinga\Module\Director\CustomVariable\CustomVariableDictionary;
 use Icinga\Module\Director\Db\AppliedServiceSetLoader;
+use Icinga\Module\Director\DirectorObject\Lookup\ServiceFinder;
 use Icinga\Module\Director\Forms\IcingaAddServiceForm;
 use Icinga\Module\Director\Forms\IcingaServiceForm;
 use Icinga\Module\Director\Forms\IcingaServiceSetForm;
@@ -20,15 +27,51 @@ use Icinga\Module\Director\Web\Controller\ObjectController;
 use Icinga\Module\Director\Web\SelfService;
 use Icinga\Module\Director\Web\Table\IcingaHostAppliedForServiceTable;
 use Icinga\Module\Director\Web\Table\IcingaHostAppliedServicesTable;
-use Icinga\Module\Director\Web\Table\IcingaHostServiceTable;
 use Icinga\Module\Director\Web\Table\IcingaServiceSetServiceTable;
-use Icinga\Module\Director\Web\Widget\HostServiceRedirector;
 
 class HostController extends ObjectController
 {
     protected function checkDirectorPermissions()
     {
-        $this->assertPermission('director/hosts');
+        $host = $this->getHostObject();
+        $auth = $this->Auth();
+        $backend = $this->backend();
+        if ($this->isServiceAction()
+            && $backend->canModifyService($host->getObjectName(), $this->getParam('service'))
+        ) {
+            return;
+        }
+        if ($this->isServicesReadOnlyAction() && $auth->hasPermission($this->getServicesReadOnlyPermission())) {
+            return;
+        }
+        if ($auth->hasPermission(Permission::HOSTS)) { // faster
+            return;
+        }
+        if ($backend->canModifyHost($host->getObjectName())) {
+            return;
+        }
+        $this->assertPermission(Permission::HOSTS); // complain about default hosts permission
+    }
+
+    protected function isServicesReadOnlyAction()
+    {
+        return in_array($this->getRequest()->getActionName(), [
+            'servicesro',
+            'findservice',
+            'invalidservice',
+        ]);
+    }
+
+    protected function isServiceAction()
+    {
+        return in_array($this->getRequest()->getActionName(), [
+            'servicesro',
+            'findservice',
+            'invalidservice',
+            'servicesetservice',
+            'appliedservice',
+            'inheritedservice',
+        ]);
     }
 
     /**
@@ -49,9 +92,10 @@ class HostController extends ObjectController
     {
         $host = $this->getHostObject();
         $this->addServicesHeader();
-        $this->addTitle($this->translate('Add Service: %s'), $host->getObjectName());
+        $this->addTitle($this->translate('Add Service to %s'), $host->getObjectName());
         $this->content()->add(
             IcingaAddServiceForm::load()
+                ->setBranch($this->getBranch())
                 ->setHost($host)
                 ->setDb($this->db())
                 ->handleRequest()
@@ -62,9 +106,11 @@ class HostController extends ObjectController
     {
         $host = $this->getHostObject();
         $this->addServicesHeader();
-        $this->addTitle($this->translate('Add Service Set: %s'), $host->getObjectName());
+        $this->addTitle($this->translate('Add Service Set to %s'), $host->getObjectName());
+
         $this->content()->add(
             IcingaServiceSetForm::load()
+                ->setBranch($this->getBranch())
                 ->setHost($host)
                 ->setDb($this->db())
                 ->handleRequest()
@@ -90,16 +136,37 @@ class HostController extends ObjectController
         ));
     }
 
-    /**
-     * @throws \Icinga\Exception\NotFoundError
-     */
     public function findserviceAction()
     {
+        $auth = $this->Auth();
         $host = $this->getHostObject();
-        $redirector = new HostServiceRedirector($host);
-        $this->redirectNow(
-            $redirector->getRedirectionUrl($this->params->get('service'))
-        );
+        $hostName = $host->getObjectName();
+        $serviceName = $this->params->get('service');
+        $info = ServiceFinder::find($host, $serviceName);
+        $backend = $this->backend();
+
+        if ($info && $auth->hasPermission(Permission::HOSTS)) {
+            $redirectUrl = $info->getUrl();
+        } elseif ($info
+            && (($backend instanceof Monitoring && $auth->hasPermission(Permission::MONITORING_HOSTS))
+                || ($backend instanceof IcingadbBackend && $auth->hasPermission(Permission::ICINGADB_HOSTS))
+            )
+            && $backend->canModifyService($hostName, $serviceName)
+        ) {
+            $redirectUrl = $info->getUrl();
+        } elseif ($auth->hasPermission($this->getServicesReadOnlyPermission())) {
+            $redirectUrl = Url::fromPath('director/host/servicesro', [
+                'name'    => $hostName,
+                'service' => $serviceName
+            ]);
+        } else {
+            $redirectUrl = Url::fromPath('director/host/invalidservice', [
+                'name'    => $hostName,
+                'service' => $serviceName,
+            ]);
+        }
+
+        $this->redirectNow($redirectUrl);
     }
 
     /**
@@ -107,59 +174,84 @@ class HostController extends ObjectController
      */
     public function invalidserviceAction()
     {
-        $this->content()->add(
-            Html::tag('p', ['class' => 'error'], sprintf(
+        if (! $this->showInfoForNonDirectorService()) {
+            $this->content()->add(Hint::error(sprintf(
                 $this->translate('No such service: %s'),
                 $this->params->get('service')
-            ))
-        );
+            )));
+        }
 
         $this->servicesAction();
     }
 
+    protected function showInfoForNonDirectorService()
+    {
+        try {
+            $api = $this->getApiIfAvailable();
+            if ($api) {
+                $name = $this->params->get('name') . '!' . $this->params->get('service');
+                $info = $api->getObject($name, 'Services');
+                if (isset($info->attrs->source_location)) {
+                    $source = $info->attrs->source_location;
+                    $this->content()->add(Hint::info(Html::sprintf(
+                        'The configuration for this object has not been rendered by'
+                        . ' Icinga Director. You can find it on line %s in %s.',
+                        Html::tag('strong', null, $source->first_line),
+                        Html::tag('strong', null, $source->path)
+                    )));
+                }
+            }
+
+            return true;
+        } catch (Exception $e) {
+            return false;
+        }
+    }
+
     /**
      * @throws \Icinga\Exception\NotFoundError
      */
     public function servicesAction()
     {
         $this->addServicesHeader();
-        $db = $this->db();
         $host = $this->getHostObject();
         $this->addTitle($this->translate('Services: %s'), $host->getObjectName());
+        $branch = $this->getBranch();
+        $hostHasBeenCreatedInBranch = $branch->isBranch() && $host->get('id');
         $content = $this->content();
-        $table = IcingaHostServiceTable::load($host)
-            ->setTitle($this->translate('Individual Service objects'));
+        $table = (new ObjectsTableService($this->db(), $this->Auth()))
+            ->setHost($host)
+            ->setBranch($branch)
+            ->setTitle($this->translate('Individual Service objects'))
+            ->removeQueryLimit();
 
         if (count($table)) {
             $content->add($table);
         }
 
-        if ($applied = $host->vars()->get($db->settings()->magic_apply_for)) {
-            if ($applied instanceof CustomVariableDictionary) {
-                $table = IcingaHostAppliedForServiceTable::load($host, $applied)
-                    ->setTitle($this->translate('Generated from host vars'));
-                if (count($table)) {
-                    $content->add($table);
-                }
-            }
-        }
-
         /** @var IcingaHost[] $parents */
         $parents = IcingaTemplateRepository::instanceByObject($this->object)
             ->getTemplatesFor($this->object, true);
         foreach ($parents as $parent) {
-            $table = IcingaHostServiceTable::load($parent)->setInheritedBy($host);
+            $table = (new ObjectsTableService($this->db(), $this->Auth()))
+                ->setBranch($branch)
+                ->setHost($parent)
+                ->setInheritedBy($host)
+                ->removeQueryLimit();
+
             if (count($table)) {
                 $content->add(
                     $table->setTitle(sprintf(
-                        'Inherited from %s',
+                        $this->translate('Inherited from %s'),
                         $parent->getObjectName()
                     ))
                 );
             }
         }
 
-        $this->addHostServiceSetTables($host);
+        if (! $hostHasBeenCreatedInBranch) {
+            $this->addHostServiceSetTables($host);
+        }
         foreach ($parents as $parent) {
             $this->addHostServiceSetTables($parent, $host);
         }
@@ -171,8 +263,10 @@ class HostController extends ObjectController
             $content->add(
                 IcingaServiceSetServiceTable::load($set)
                     // ->setHost($host)
+                    ->setBranch($branch)
                     ->setAffectedHost($host)
                     ->setTitle($title)
+                    ->removeQueryLimit()
             );
         }
 
@@ -184,16 +278,101 @@ class HostController extends ObjectController
         }
     }
 
+    /**
+     * Hint: this duplicates quite some logic from servicesAction. We might want
+     *       to clean this up, but as soon as we store fully resolved Services this
+     *       will be obsolete anyways
+     *
+     * @throws \Icinga\Exception\NotFoundError
+     * @throws \Icinga\Security\SecurityException
+     * @throws \Icinga\Exception\MissingParameterException
+     */
+    public function servicesroAction()
+    {
+        $this->assertPermission($this->getServicesReadOnlyPermission());
+        $host = $this->getHostObject();
+        $service = $this->params->getRequired('service');
+        $db = $this->db();
+        $branch = $this->getBranch();
+        $this->controls()->setTabs(new Tabs());
+        $this->addSingleTab($this->translate('Configuration (read-only)'));
+        $this->addTitle($this->translate('Services on %s'), $host->getObjectName());
+        $content = $this->content();
+
+        $table = (new ObjectsTableService($db, $this->Auth()))
+            ->setHost($host)
+            ->setBranch($branch)
+            ->setReadonly()
+            ->highlightService($service)
+            ->setTitle($this->translate('Individual Service objects'));
+
+        if (count($table)) {
+            $content->add($table);
+        }
+
+        /** @var IcingaHost[] $parents */
+        $parents = IcingaTemplateRepository::instanceByObject($this->object)
+            ->getTemplatesFor($this->object, true);
+        foreach ($parents as $parent) {
+            $table = (new ObjectsTableService($db, $this->Auth()))
+                ->setReadonly()
+                ->setBranch($branch)
+                ->setHost($parent)
+                ->highlightService($service)
+                ->setInheritedBy($host);
+            if (count($table)) {
+                $content->add(
+                    $table->setTitle(sprintf(
+                        'Inherited from %s',
+                        $parent->getObjectName()
+                    ))
+                );
+            }
+        }
+
+        $this->addHostServiceSetTables($host);
+        foreach ($parents as $parent) {
+            $this->addHostServiceSetTables($parent, $host, $service);
+        }
+
+        $appliedSets = AppliedServiceSetLoader::fetchForHost($host);
+        foreach ($appliedSets as $set) {
+            $title = sprintf($this->translate('%s (Applied Service set)'), $set->getObjectName());
+
+            $content->add(
+                IcingaServiceSetServiceTable::load($set)
+                    // ->setHost($host)
+                    ->setBranch($branch)
+                    ->setAffectedHost($host)
+                    ->setReadonly()
+                    ->highlightService($service)
+                    ->setTitle($title)
+            );
+        }
+
+        $table = IcingaHostAppliedServicesTable::load($host)
+            ->setReadonly()
+            ->highlightService($service)
+            ->setTitle($this->translate('Applied services'));
+
+        if (count($table)) {
+            $content->add($table);
+        }
+    }
+
     /**
      * @param IcingaHost $host
      * @param IcingaHost|null $affectedHost
      */
-    protected function addHostServiceSetTables(IcingaHost $host, IcingaHost $affectedHost = null)
+    protected function addHostServiceSetTables(IcingaHost $host, IcingaHost $affectedHost = null, $roService = null)
     {
         $db = $this->db();
         if ($affectedHost === null) {
             $affectedHost = $host;
         }
+        if ($host->get('id') === null) {
+            return;
+        }
 
         $query = $db->getDbAdapter()->select()
             ->from(
@@ -213,12 +392,16 @@ class HostController extends ObjectController
         /** @var IcingaServiceSet $set*/
         foreach ($sets as $name => $set) {
             $title = sprintf($this->translate('%s (Service set)'), $name);
-            $this->content()->add(
-                IcingaServiceSetServiceTable::load($set)
-                    ->setHost($host)
-                    ->setAffectedHost($affectedHost)
-                    ->setTitle($title)
-            );
+            $table = IcingaServiceSetServiceTable::load($set)
+                ->setHost($host)
+                ->setBranch($this->getBranch())
+                ->setAffectedHost($affectedHost)
+                ->removeQueryLimit()
+                ->setTitle($title);
+            if ($roService) {
+                $table->setReadonly()->highlightService($roService);
+            }
+            $this->content()->add($table);
         }
     }
 
@@ -249,6 +432,7 @@ class HostController extends ObjectController
         $this->content()->add(
             IcingaServiceForm::load()
                 ->setDb($db)
+                ->setBranch($this->getBranch())
                 ->setHost($host)
                 ->setApplyGenerated($parent)
                 ->setObject($service)
@@ -289,6 +473,7 @@ class HostController extends ObjectController
 
         $form = IcingaServiceForm::load()
             ->setDb($db)
+            ->setBranch($this->getBranch())
             ->setHost($host)
             ->setInheritedFrom($from->getObjectName())
             ->setObject($service)
@@ -366,6 +551,7 @@ class HostController extends ObjectController
 
         $form = IcingaServiceForm::load()
             ->setDb($db)
+            ->setBranch($this->getBranch())
             ->setHost($host)
             ->setServiceSet($set)
             ->setObject($service)
@@ -406,20 +592,19 @@ class HostController extends ObjectController
     {
         $host = $this->object;
         try {
-            $mon = $this->monitoring();
-            if ($host->isObject()
-                && $mon->isAvailable()
-                && $mon->hasHost($host->getObjectName())
+            $backend = $this->backend();
+            if ($host instanceof IcingaHost
+                && $host->isObject()
+                && $backend->hasHost($host->getObjectName())
             ) {
-                $this->actions()->add(Link::create(
-                    $this->translate('Show'),
-                    'monitoring/host/show',
-                    ['host' => $host->getObjectName()],
-                    [
-                        'class'            => 'icon-globe critical',
-                        'data-base-target' => '_next'
-                    ]
-                ));
+                $this->actions()->add(
+                    Link::create(
+                        $this->translate('Show'),
+                        $backend->getHostUrl($host->getObjectName()),
+                        null,
+                        ['class' => 'icon-globe critical', 'data-base-target' => '_next']
+                    )
+                );
 
                 // Intentionally placed here, show it only for deployed Hosts
                 $this->addOptionalInspectLink();
@@ -431,7 +616,7 @@ class HostController extends ObjectController
 
     protected function addOptionalInspectLink()
     {
-        if (! $this->hasPermission('director/inspect')) {
+        if (! $this->hasPermission(Permission::INSPECT)) {
             return;
         }
 
@@ -451,11 +636,25 @@ class HostController extends ObjectController
     }
 
     /**
-     * @return IcingaHost
+     * @return ?IcingaHost
      */
     protected function getHostObject()
     {
-        /** @var IcingaHost $this->object */
+        if ($this->object !== null) {
+            assert($this->object instanceof IcingaHost);
+        }
         return $this->object;
     }
+
+    /**
+     * Get readOnly permission of the service for the current backend
+     *
+     * @return string permission
+     */
+    protected function getServicesReadOnlyPermission(): string
+    {
+        return $this->backend() instanceof IcingadbBackend
+            ? Permission::ICINGADB_SERVICES_RO
+            : Permission::MONITORING_SERVICES_RO;
+    }
 }
