@@ -3,6 +3,8 @@
 namespace Icinga\Module\Director\Data\Db;
 
 use Icinga\Data\Db\DbConnection as IcingaDbConnection;
+use Icinga\Module\Director\Db\DbUtil;
+use RuntimeException;
 use Zend_Db_Expr;
 
 class DbConnection extends IcingaDbConnection
@@ -17,13 +19,23 @@ class DbConnection extends IcingaDbConnection
         return $this->getDbType() === 'pgsql';
     }
 
+    /**
+     * @deprecated
+     * @param ?string $binary
+     * @return Zend_Db_Expr|Zend_Db_Expr[]|null
+     */
     public function quoteBinary($binary)
     {
-        if ($this->isPgsql()) {
-            return new Zend_Db_Expr("'\\x" . bin2hex($binary) . "'");
+        return DbUtil::quoteBinaryLegacy($binary, $this->getDbAdapter());
+    }
+
+    public function binaryDbResult($value)
+    {
+        if (is_resource($value)) {
+            return stream_get_contents($value);
         }
 
-        return $binary;
+        return $value;
     }
 
     public function hasPgExtension($name)
