@@ -36,11 +36,13 @@ class IcingaObjectImports implements Iterator, Countable, IcingaConfigRenderer
         $this->object = $object;
     }
 
+    #[\ReturnTypeWillChange]
     public function count()
     {
         return count($this->imports);
     }
 
+    #[\ReturnTypeWillChange]
     public function rewind()
     {
         $this->position = 0;
@@ -61,6 +63,7 @@ class IcingaObjectImports implements Iterator, Countable, IcingaConfigRenderer
      * @return IcingaObject|null
      * @throws \Icinga\Exception\NotFoundError
      */
+    #[\ReturnTypeWillChange]
     public function current()
     {
         if (! $this->valid()) {
@@ -72,16 +75,19 @@ class IcingaObjectImports implements Iterator, Countable, IcingaConfigRenderer
         );
     }
 
+    #[\ReturnTypeWillChange]
     public function key()
     {
         return $this->idx[$this->position];
     }
 
+    #[\ReturnTypeWillChange]
     public function next()
     {
         ++$this->position;
     }
 
+    #[\ReturnTypeWillChange]
     public function valid()
     {
         return array_key_exists($this->position, $this->idx);
@@ -192,7 +198,7 @@ class IcingaObjectImports implements Iterator, Countable, IcingaConfigRenderer
         if (is_array($import)) {
             foreach ($import as $i) {
                 // Gracefully ignore null members or empty strings
-                if (! $i instanceof $class && strlen($i) === 0) {
+                if (! $i instanceof $class && ($i === null || strlen($i) === 0)) {
                     continue;
                 }
 
@@ -232,6 +238,7 @@ class IcingaObjectImports implements Iterator, Countable, IcingaConfigRenderer
     {
         $list = [];
         foreach ($this->listImportNames() as $name) {
+            $name = (string) $name;
             $list[$name] = $this->getObject($name);
         }
 
@@ -307,7 +314,7 @@ class IcingaObjectImports implements Iterator, Countable, IcingaConfigRenderer
             $this->imports = array_combine($keys, $keys);
         }
 
-        $this->cloneStored();
+        $this->setBeingLoadedFromDb();
         return $this;
     }
 
@@ -354,12 +361,12 @@ class IcingaObjectImports implements Iterator, Countable, IcingaConfigRenderer
             ]);
         }
 
-        $this->cloneStored();
+        $this->setBeingLoadedFromDb();
 
         return true;
     }
 
-    protected function cloneStored()
+    public function setBeingLoadedFromDb()
     {
         $this->storedNames = $this->listImportNames();
         $this->modified = false;
