@@ -2,11 +2,16 @@
 
 namespace Icinga\Module\Director\Web\Controller;
 
+use gipfl\Translation\StaticTranslator;
 use Icinga\Application\Benchmark;
+use Icinga\Application\Modules\Module;
 use Icinga\Data\Paginatable;
 use Icinga\Exception\NotFoundError;
 use Icinga\Exception\ProgrammingError;
-use Icinga\Module\Director\Monitoring;
+use Icinga\Module\Director\Integration\Icingadb\IcingadbBackend;
+use Icinga\Module\Director\Integration\BackendInterface;
+use Icinga\Module\Director\Integration\MonitoringModule\Monitoring;
+use Icinga\Module\Director\ProvidedHook\Icingadb\IcingadbSupport;
 use Icinga\Module\Director\Web\Controller\Extension\CoreApi;
 use Icinga\Module\Director\Web\Controller\Extension\DirectorDb;
 use Icinga\Module\Director\Web\Controller\Extension\RestApi;
@@ -17,7 +22,6 @@ use Icinga\Web\UrlParams;
 use InvalidArgumentException;
 use gipfl\IcingaWeb2\Translator;
 use gipfl\IcingaWeb2\Link;
-use gipfl\Translation\TranslationHelper;
 use gipfl\IcingaWeb2\Widget\ControlsAndContent;
 use gipfl\IcingaWeb2\Controller\Extension\ControlsAndContentHelper;
 use gipfl\IcingaWeb2\Zf1\SimpleViewRenderer;
@@ -36,8 +40,8 @@ abstract class ActionController extends Controller implements ControlsAndContent
     /** @var UrlParams Hint for IDE, somehow does not work in web */
     protected $params;
 
-    /** @var Monitoring */
-    private $monitoring;
+    /** @var BackendInterface */
+    private $backend;
 
     /**
      * @throws SecurityException
@@ -60,7 +64,7 @@ abstract class ActionController extends Controller implements ControlsAndContent
 
     protected function initializeTranslator()
     {
-        TranslationHelper::setTranslator(new Translator('director'));
+        StaticTranslator::set(new Translator('director'));
     }
 
     public function getAuth()
@@ -219,7 +223,7 @@ abstract class ActionController extends Controller implements ControlsAndContent
             // Hint -> $this->view->compact is the only way since v2.8.0
             if ($this->view->compact || $this->getOriginalUrl()->getParam('view') === 'compact') {
                 if ($this->view->controls) {
-                    $this->controls()->getAttributes()->add('style', 'display: none;');
+                    $this->controls()->getAttributes()->add('class', 'compact');
                 }
             }
         } else {
@@ -240,14 +244,18 @@ abstract class ActionController extends Controller implements ControlsAndContent
     }
 
     /**
-     * @return Monitoring
+     * @return BackendInterface
      */
-    protected function monitoring()
+    protected function backend(): BackendInterface
     {
-        if ($this->monitoring === null) {
-            $this->monitoring = new Monitoring;
+        if ($this->backend === null) {
+            if (Module::exists('icingadb') && IcingadbSupport::useIcingaDbAsBackend()) {
+                $this->backend = new IcingadbBackend();
+            } else {
+                $this->backend = new Monitoring($this->getAuth());
+            }
         }
 
-        return $this->monitoring;
+        return $this->backend;
     }
 }
